/////////////////////////////////////////////////////
// Dumps the Memory of a process from a start address
// for a certain length to a filename
/////////////////////////////////////////////////////
#include <stdio.h>
#include <windows.h>
#include "pluginsupport.h"

HANDLE privateHeap;
char* lastErrorMessage;

// DLL entry point
BOOL WINAPI DllMain(HINSTANCE hDLL, DWORD dwReason, LPVOID Reserved)
{
	switch (dwReason)
	{
		case DLL_PROCESS_ATTACH:
		{
			privateHeap = HeapCreate(NULL, 1000000, 0);

			lastErrorMessage = (char*)HeapAlloc(privateHeap, NULL, 1000);
			ZeroMemory(lastErrorMessage, 1000);

			break;
		}
		case DLL_PROCESS_DETACH:
		{
			HeapDestroy(privateHeap);
			break;
		}
	}

	return true;
}

////////////////////////////////////////////
// GetLastPluginError
//
// if an error occurred, used by the process
// patcher to retrieve the error message
////////////////////////////////////////////
char* GetLastPluginError(void)
{
	return lastErrorMessage;
}

///////////////////////////////////////////////
// SetLastPluginError
//
// set the error message should an error occurs
///////////////////////////////////////////////
void SetLastPluginError(char* errorMsg)
{
	lstrcpy(lastErrorMessage, errorMsg);
}

///////////////////////////////////////////////////
// GetPluginVersionInformation
//
// version API for retrieval by the process patcher
///////////////////////////////////////////////////
bool GetPluginVersionInformation(PLUGIN_VERSION_INFORMATION* pluginVersionInfo)
{
	pluginVersionInfo->copyrightInformation =  "Copyright 1999 thewd";
	pluginVersionInfo->descriptionInformation = "Dump Process Memory";
	pluginVersionInfo->majorVersion = 1;
	pluginVersionInfo->minorVersion = 0;

	return true;
}

/////////////////////////
// ExecutePlugin
//
// execute the plugin API
/////////////////////////
bool ExecutePlugin(PLUGIN_EXECUTE_INFORMATION* pluginExecuteInfo)
{
	if (pluginExecuteInfo->numberOfArguments != 3)
	{
		SetLastPluginError("(Usage: <output filename> <memory offset> <number of bytes>)");
		return false;
	}

	char* outputFileName = (char*)HeapAlloc(privateHeap, NULL, 1000);
	lstrcpy(outputFileName, pluginExecuteInfo->pluginArguments[0]);
	unsigned int fileOffset;
	unsigned int fileLength;

	// get file offset;
	if ((pluginExecuteInfo->pluginArguments[1][0] == '0') &&
		((pluginExecuteInfo->pluginArguments[1][1] == 'x') ||
		(pluginExecuteInfo->pluginArguments[1][1] == 'X')))
	{
		sscanf(pluginExecuteInfo->pluginArguments[1], "%x", &fileOffset);
	}
	else
	{
		sscanf(pluginExecuteInfo->pluginArguments[1], "%d", &fileOffset);
	}

	// get file length
	if ((pluginExecuteInfo->pluginArguments[2][0] == '0') &&
		((pluginExecuteInfo->pluginArguments[2][1] == 'x') ||
		(pluginExecuteInfo->pluginArguments[2][1] == 'X')))
	{
		sscanf(pluginExecuteInfo->pluginArguments[2], "%x", &fileLength);
	}
	else
	{
		sscanf(pluginExecuteInfo->pluginArguments[2], "%d", &fileLength);
	}

	// check for valid parameters
	if ((outputFileName[0] == NULL) ||
		(fileOffset == 0) ||
		(fileLength == 0))
	{
		SetLastPluginError("Error Parsing Plugin Arguments");
		return false;
	}

	MessageBox(NULL, "Press OK when ready to dump memory",
		"Dump Memory", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL);

	unsigned char* buffer = (unsigned char*)HeapAlloc(privateHeap, NULL, fileLength);

	HANDLE processHandle = OpenProcess(PROCESS_VM_READ, true,
		pluginExecuteInfo->processId);

	if (processHandle == NULL)
	{
		SetLastPluginError("Error Opening Process");
		return false;
	}

	unsigned long bytesRead = 0;

	if (ReadProcessMemory(processHandle, (void*)fileOffset,
		buffer, fileLength, &bytesRead) == 0)
	{
		SetLastPluginError("Error Reading Memory of Process");
		return false;
	}

	CloseHandle(processHandle);

	unsigned long actuallyWritten;

	HANDLE fileHandle = CreateFile(outputFileName, GENERIC_WRITE, 0,
		NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (fileHandle == INVALID_HANDLE_VALUE)
	{
		SetLastPluginError("Error Writting Memory To Disk");
		return false;
	}
	WriteFile(fileHandle, buffer, fileLength, &actuallyWritten, NULL);
	CloseHandle(fileHandle);
	if (actuallyWritten != fileLength)
	{
		SetLastPluginError("Error Writting Memory To Disk");
		return false;
	}

	HeapFree(privateHeap, NULL, buffer);

	MessageBox(NULL, "Successfully written memory to disk",
		"Dump Memory", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL);

	return true;
}